unit Ficestream;

interface

Uses Classes, Windows, SysUtils, DebugLog;

Type
  TFiceFileStream = class(TFileStream)
    public
      Destructor Destroy; Override;
  end;
  TFiceResourceStream = class(TResourceStream)
    public
      Destructor Destroy; Override;
  end;
  EFiceException = class(Exception);

function InputStream(FName:ShortString): TStream;
function FiceExists(FName:ShortString): Boolean;
function FiceInputStream(FName:ShortString): TStream;
function FiceProcAddress(FuncName:ShortString): Pointer;
function FiceProcAddressE(FuncName:ShortString): Pointer;
procedure FiceProcSort(Cmp:TListSortCompare);

var
  DLLs:     TStringList;

implementation

var
  Garbage:  TList;

function InputStream(Fname:ShortString): TStream;
var
I:        Integer;
begin
Try
If FileExists(ExtractFilePath(ParamStr(0))+FName) then begin
  Result := TFileStream.Create(ExtractFilePath(ParamStr(0))+FName, fmOpenRead or fmShareDenyNone);
  Log('FiceStream '+FName+' located from external file'); Exit;
end else begin
  For I := 0 to DLLs.Count-1 do begin
    Try
      Result := TResourceStream.Create(Integer(DLLs.Objects[I]),FName,RT_RCDATA);
    Except
      Result := nil;
    end;
    If Result<>nil then begin
      Log('FiceStream '+FName+' located as resource in library '+DLLs[I]);
      Exit;
    end;
  end;
end;
Except
Result := nil;
end;
Log('FiceStream '+FName+' not located!');
end;

function FiceInputStream(FName:ShortString): TStream;
var
I:        Integer;
begin
Try
If FileExists(ExtractFilePath(ParamStr(0))+FName) then begin
  Result := TFiceFileStream.Create(ExtractFilePath(ParamStr(0))+FName, fmOpenRead or fmShareDenyNone);
  Log('FiceStream '+FName+' located from external file');
end else begin
  For I := 0 to DLLs.Count-1 do begin
    Try
      Result := TFiceResourceStream.Create(Integer(DLLs.Objects[I]),FName,RT_RCDATA);
    Except
      Result.Free;
      Result := nil;
    end;
    If Result<>nil then begin
      Log('FiceStream '+FName+' located as resource in library '+DLLs[I]);
      Exit;
    end;
  end;
end;
Except
Result := nil;
Log('FiceStream '+FName+' not located!');
end;
end;

function FiceExists(FName:ShortString): Boolean;
var
Res:  TResourceStream;
I:    Integer;
begin
Result := True;
If FileExists(ExtractFilePath(ParamStr(0))+FName) then Exit;
For I := 0 to DLLs.Count-1 do begin
  Result := True;
  Try
    Res := TResourceStream.Create(Integer(DLLs[I]), FName, RT_RCDATA);
  Except
    On E: EResNotFound do Result := False;
    On E: Exception do Raise E;
  end;
  Res.Free;
  If Result=True then Exit;
end;
end;

Destructor TFiceFileStream.Destroy;
begin
Garbage.Remove(Self);
Inherited Destroy;
end;

Destructor TFiceResourceStream.Destroy;
begin
Garbage.Remove(Self);
Inherited Destroy;
end;

function FiceProcAddress(FuncName:ShortString): Pointer;
var
I:      Integer;
NT:     Array[0..255] of Char;
begin
Result := nil;
For I := 0 to DLLs.Count-1 do begin
  StrPCopy(NT,FuncName);
  Result := GetProcAddress(Integer(DLLs.Objects[I]),NT);
  If Result<>nil then begin
    Log('FiceProc: Function '+FuncName+' found in library '+DLLs[I]);
    Exit;
  end;
end;
Log('FiceProc: Couldn''t locate function '+FuncName);
end;

function FiceProcAddressE(FuncName:ShortString): Pointer;
begin
Result := FiceProcAddress(FuncName);
If Result=nil then Raise EFiceException.Create('FiceProc: Couldn''t locate function '+FuncName);
end;

procedure GarbageCollect;
var
I:      Integer;
begin
Log('FiceStream garbage collecting');
For I := 0 to Garbage.Count-1 do
  TObject(Garbage[I]).Free;
Garbage.Free;
Log('FiceProc cleaning up');
For I := 0 to DLLs.Count-1 do
  FreeLibrary(Integer(DLLs.Objects[I]));
DLLs.Free;
Log('FiceStream & FiceProc finalized');
end;

procedure FiceProcSort(Cmp:TListSortCompare);
var
List:     TList;
NewSL:    TStringList;
I,J:      Integer;
begin
Log('FiceProc: Sorting source libraries');
List := TList.Create;
For I := 0 to DLLs.Count-1 do List.Add(DLLs.Objects[I]);
List.Sort(Cmp);
NewSL := TStringList.Create;
For I := 0 to List.Count-1 do begin
  J := DLLs.IndexOfObject(List[I]);
  NewSL.AddObject(DLLs[J], DLLs.Objects[J]);
end;
DLLs.Free;
DLLs := NewSL;
List.Free;
end;

procedure InitFice;
var
SR:       TSearchRec;
I:        Integer;
DLL:      THandle;
begin
Garbage := TList.Create;
Log('FiceStream initialized and ready');
DLLs := TStringList.Create;
I := FindFirst(ExtractFilePath(ParamStr(0))+'*.DLL',0,SR);
While I=0 do begin
  DLL := LoadLibrary(PChar(SR.Name));
  If DLL<>0 then DLLs.AddObject(SR.Name,Pointer(DLL));
  I := FindNext(SR);
end;
FindClose(SR);

I := FindFirst(ExtractFilePath(ParamStr(0))+'*.EXE',0,SR);
While I=0 do begin
  DLL := LoadLibrary(PChar(ExtractFilePath(ParamStr(0))+SR.Name));
  If DLL<>0 then DLLs.AddObject(SR.Name,Pointer(DLL));
  I := FindNext(SR);
end;

Log('FiceProc initialized and ready');
end;

initialization
InitFice;
finalization
GarbageCollect;
end.
